import zipfile
import os.path
import sha

import usage

def ComputeZip(opts, args):
    """Compute checksums of the files inside an archive.

    This function will scan all the files within a ZIP and compute SHA-1 checksums
    for them. Optionally, it will also print a MAME-ready ROM_START()/ROM_END()
    definition that loads the files.
    """
    if args[0] == 'd':
        DriverMode = 1
    else:
        DriverMode = 0

    # Shift by one the parameters
    del args[0]

    if len(args) == 0:
        print "No file(s) specified"
        return

    # process all the arguments
    for zipFileName in args:
        try:
            z = zipfile.ZipFile(zipFileName)
        except IOError:
            print "ERROR: Cannot open %s" % zipFileName
            continue
        except zipfile.BadZipfile:
            print "ERROR: %s is not a ZIP file" % zipFileName
            continue

        if DriverMode:
            print "ROM_START( %s )" % os.path.splitext(os.path.basename(zipFileName))[0]
        else:
            print "%s" % zipFileName

        # Process all the files inside the archive
        for fileName in z.namelist():

            # Skip files of 0 bytes (directories for instance)
            if z.getinfo(fileName).file_size == 0:
                continue

            # Calculate SHA-1 of the file
            sha1 = sha.new(z.read(fileName)).hexdigest()

            if DriverMode:
                formattedFileName = "%-15s" % ('"' + os.path.basename(fileName).lower() + '",')
                
                print '\tROM_LOAD( %s 0x000000, 0x%06x, CRC(%08x) SHA1(%s) ) ' % (
                    formattedFileName, z.getinfo(fileName).file_size,
                    z.getinfo(fileName).CRC, sha1)
            else:
                print "  %s" % fileName
                print "    %08x   %s" % (z.getinfo(fileName).CRC, sha1)

        if DriverMode:
            print "ROM_END"
            print
                
        z.close()
