# DriverRegex.py - contains all the regular expression used to parse a MAME driver
import re;

sSPACE = r"\s*";  # match any whitespace
sALL = ".*";      # match everything

sLOADSTRINGS = (
    "(?:"
    "ROM_LOAD|"
    "ROMX_LOAD|"
    "ROM_LOAD_128K|"
    "ROM_LOAD_256K|"
    "ROM_LOAD_512K|"
    "ROM_LOAD16_BYTE|"
    "ROM_LOAD16_WORD|"
    "ROM_LOAD16_WORD_SWAP|"
    "ROM_LOAD32_BYTE|"
    "ROM_LOAD32_WORD|"
    "ROM_LOAD32_WORD_SWAP|"
    "ROM_LOAD64_WORD|"   # rungun.c
    "ROM_LOADTILE_WORD|"  # mystwarr.c
    "ROM_LOADTILE_BYTE|"  # mystwarr.c
    "ROM_LOAD_HS|"
    "ROM_LOAD_NIB_LOW|"
    "ROM_LOAD_NIB_HIGH|"
    "NEO_SFIX_32K|"        #neogeo.c
    "NEO_SFIX_64K|"        #neogeo.c
    "NEO_SFIX_128K|"       #neogeo.c
    "NEO_BIOS_SOUND_64K|"  #neogeo.c
    "NEO_BIOS_SOUND_128K|" #neogeo.c
    "NEO_BIOS_SOUND_256K|" #neogeo.c
    "NEO_BIOS_SOUND_512K|" #neogeo.c
    "DATA_LOAD_E_128K|"  # namcos2.c
    "DATA_LOAD_O_128K|"  # namcos2.c
    "DATA_LOAD_E_256K|"  # namcos2.c
    "DATA_LOAD_O_256K|"  # namcos2.c
    "DATA_LOAD_E_512K|"  # namcos2.c
    "DATA_LOAD_O_512K)"  # namcos2.c
    );

sROMSTART = (
    r"^ROM_START"
    r"\s*"
    r"\("
    r"\s*"
    r"([^)\s]+)"  # game name (everything but space and parenthesis)
    r"\s*"
    r"\)"
    r".*"
    );


sROMLOAD = (
    ".*"
    + sLOADSTRINGS
    + sSPACE +
    r"\("
    + sSPACE +
    "\"([^\"]+)\""  # filename between quotes [1]
    + sSPACE +
    ","
    + sSPACE +
    "(?:[^,]*,)?"  # skip to next comma (if exists) - for NEO_SHIT()
    "(?:[^,]*,)?" # skip to next comma (if exists)
    + sSPACE +
    r"(?:BADCRC\s*\()?" # optional BADCRC
    + sSPACE +
    "(?:0[xX])?"		  # crc hex start and/or single 0
    "([0-9a-fA-F]+)"    # crc [2]
    + sSPACE +
    r"(?:\))?"  # optional close parenthesis for BADCRC
    + sSPACE +
    r"[\),]"                 # after the CRC part, there is either a parenthesis
                             # or a comma (very important)
    + sALL
    );

sROMLOAD2 = (
    ".*"
    + sLOADSTRINGS
    + sSPACE + 
    "\\("
    + sSPACE +
    "\"[^\"]+\""             # filename between quotes
    + sSPACE + 
    ","
    + sSPACE +
    "(?:[^,]*,)?"            # skip to next comma (if exists) - for NEO_SHIT()
    "(?:[^,]*,)?"            # skip to next comma (if exists)
    + sSPACE +
    "("                      # [1] the whole crc section
    r"(?:BADCRC\s*\()?"      # optional BADCRC
    + sSPACE +
    "(?:0[xX])?"		     # crc hex start and/or single 0
    "(?:[0-9a-fA-F]+)"       # crc
    + sSPACE +
    r"(?:\))?"               # optional close parenthesis for BADCRC
    ")"
    + sSPACE +
    r"[\),]"                 # after the CRC part, there is either a parenthesis
                             # or a comma (very important)
    );

sROMEND = (
    r"^ROM_END"
    r".*"
    );

sGAME = (
    "(?:GAME|GAMEX)"  # cannot match from start of the line (8080bw.c)
    r"\s*"
    "\\("
    r"\s*"
    "[^,]*,"   # skip date
    r"\s*"
    r"([^,\s]+)"  # game name [2]
    r"\s*"
    ","
    r"\s*"
    r"([^,\s]+)"  # parent name [2]
    r"\s*"
    ","
    r"\s*"
    r".*"
    );

# Compile the regular expressions for further use
rROMSTART = re.compile(sROMSTART);
rROMLOAD  = re.compile(sROMLOAD);
rROMLOAD2 = re.compile(sROMLOAD2);
rROMEND   = re.compile(sROMEND);
rGAME     = re.compile(sGAME);
